## Precept 9
## November 17 2016
## Simone Zhang 
## *Credit as always to previous preceptors*

#######################################################################
## dplyr 
## Using dplyr to perform data manipulations and painlessly summarize 
#######################################################################

library(dplyr)
# We're going to use the diamonds data set, which is included in the ggplot package 
library(ggplot2)

# Let's take a look at what these data look like 
head(diamonds)
diamonds$cut <- as.factor(diamonds$cut) # let's convert cut to a factor variable 
levels(diamonds$cut) # we can check the levels of the factor variable

# Say we want to calculate the mean of carat, depth, and price 
# How would we do this in base R? 
diamonds_new_base <- apply(diamonds[, c("carat", "depth", "price")], MARGIN = 2, FUN = mean)
diamonds_new_base

# How might we do this in dplyr?
diamonds_new_dplyr <- summarise_each(select(diamonds, carat, depth, price),funs(mean))
diamonds_new_dplyr

# Often we are interested in subgroups
# For instance, what if we want to calculate the mean and standard deviation of carat weight and 
# average price for diamonds by cut?

# How would we do this normally in base R? 

# Now for dyplyr magic with the piping operator! 
diamond_mean_sd <- diamonds %>%
  group_by(cut) %>% 
  select(carat, price, depth) %>% # select columns
  summarise_each(funs(mean, sd, var)) # summarize columns with the provided columns 
  
# What if we wanted to find the mean price per carat by diamond colour?
diamond_price_carat <- diamonds %>%
  mutate(price_per_carat = price / carat) %>% # create new variable that is price per carat
  group_by(color) %>%
  summarise(mean_price = mean(price_per_carat)) %>% #summarize data based on mean price 
  arrange(desc(mean_price)) # sort by mean price 

diamond_price_carat

# Magic!! 

#########################
## Checking assumptions #
#########################
library(foreign)

cc <- read.dta("http://isites.harvard.edu/fs/docs/icb.topic636824.files/Section%2010/ccarddata.dta")
names(cc) <- c("age", "income", "ccexpend", "homeowner")

head(cc)
summary(cc)

# No Missing Data
# People ages 20-55
# Everyone spent at least some money with credit cards
# 37.5% own their homes
# *monthly* household income in thousands

lm.cc <- lm(ccexpend ~ income + homeowner + age, data = cc)
summary(lm.cc)

############################################### 
## Simulate homoskedastic data for comparison #
###############################################
n <- 100
x <- seq(-5, 5, length.out = n)
y <- 2 + 3*x + rnorm(n, 0, 1)  # This is the true data generating process 

reg <- lm(y ~ x)
summary(reg)

# Note that here we're employing a general strategy of
# simulating data where we know the truth as a point of reference

######################## 
## Heteroskedasticity ##
########################
library(MASS)

# why is it a problem?  Our estimated SEs will be biased
# and OLS is no longer efficient.  

## Let's do some visual diagnostics 
## Let's plot the residuals on fitted values using base plot 
par(mfrow = c(2,1))
plot(lm.cc, 1)
plot(reg, 1)

## plot residuals vs data
par(mfrow = c(3,1))
plot(residuals(lm.cc) ~ cc$income, pch = 19, main = "Residuals on Income")
plot(residuals(lm.cc) ~ cc$homeowner, xlim = c(-.2, 1.2), pch = 18, main = "Residuals on Homeowner")
plot(residuals(lm.cc) ~ cc$age, pch = 19, main = "Residuals on Age")


##############
## Normality #
##############
## Three kinds of residuals
resid(lm.cc) # actual residuals
rstandard(lm.cc) # standardized residuals 
student_resid_cc <- studres(lm.cc) # studentized residuals. We'll run our diagnostics on these.
student_resid_sim <- studres(reg) # studentized residuals from the simulated data

# We can examining the distribution of the studentized residuals 
# Do the residuals look normally distributed?
ggplot() + geom_density(aes(x = student_resid_cc)) +
  xlab("Studentized Residuals") +
  ggtitle("Distribution of Studentized Residuals - Credit Card Data")

# For comparison, this is what the studentized residuals from our simulated data look like 
ggplot() + geom_density(aes(x = student_resid_sim)) +
  xlab("Studentized Residuals") +
  ggtitle("Distribution of Studentized Residuals - Simulated Data")

## Another way to check for normality is via a q-q plot 
## Q-Q plot
par(mfrow = c(2,1))
plot(lm.cc, which = 2, main = "Q-Q plot for Credit Card Data")
plot(reg, which = 2, main = "Q-Q plot Simulated Data")
## plotting quantiles of standardized residuals against
## normally distributed quantiles

###############
## Linearity ##
###############

# why do we care?  Bias in the estimated regression function
# which means poor prediction/inference.  If the data is 
# very non-linear, in the most important way the model 
# is not good.  

# gam plots
library(mgcv)

gam.cc <- gam(ccexpend ~ s(income) + s(age) + homeowner, data = cc) 

# s() lets the function choose a smooth functional form that
# minimizes deviations from the surface without becoming too wiggly
# (a penalty for the variance-bias tradeoff is written into the function)

summary(gam.cc)
plot(gam.cc)

# What do these numbers mean?

# edf: equivalent degrees of freedom. When edf=number of independent
# variables, linear relationship captures effect. When edf > k, smooth
# surface deviates from linear.  Think about edf > k  as having to use
# an additional variable to describe a smooth regression surface.
# (This gam eats up two DF estimating the smooth curve for income, 
# suggesting that we need to consider one extra term).

# Notice the approximate significance of smooth terms 
# F value and associated p-value: probability that variable would
# have this strong of an effect under the null hypothesis that there
# is no relationship (treat with skepticism, better off presenting/graphing
# confidence bands)

inc.sq <- cc$income ^ 2 # Adding a quadratic term for income
log.inc <- log(cc$income) # Logging income
gam.cc.2 <- gam(ccexpend ~ s(income) + s(inc.sq) + s(age) + homeowner, data = cc)
summary(gam.cc.2)

 
gam.cc.3 <- gam(ccexpend ~ s(log.inc) + s(age) + homeowner, data = cc)
summary(gam.cc.3)
plot(gam.cc.3)


# GCV: generalized cross-validation score, ability to predict out of
# sample.  We can compare these across models and try to minimize
# this value.

# often it makes more sense to plot the gam objects to see the
# partial relationships for each variable.

par(mfrow = c(2,1))

# new model!  ...income gets a nice quadratic fit.
plot(gam.cc.2)

# Here's the new resid vs fitted plot
ggplot(data.frame(Fitted = predict(gam.cc.2),
                  Residuals = residuals(gam.cc.2)),
       aes(x = Fitted, y = Residuals)) +
  geom_point() +
  stat_smooth() +
  ggtitle("Residual plot for new GAM fit")

# note: we can also look at GAM fits in multiple dimensions
par(mfrow = c(1,1))
vis.gam(gam.cc, view = c("income","age"), 
        type = "response", phi = 20, theta = -45)

# take out sections of the surface that are too far from given data points 
vis.gam(gam.cc, view = c("income","age"), 
        type = "response", phi = 20, theta = -45, too.far = .1) 

# here's our revised model
lm.cc2 <- lm(ccexpend ~ income + I(income^2) + 
               homeowner + age, data = cc)
summary(lm.cc2)

# Reminder: here's what the residuals vs fitted plot looked like for our original model 
ggplot(data.frame(Fitted = predict(lm.cc),
                  Residuals = residuals(lm.cc)),
       aes(x = Fitted, y = Residuals)) +
  geom_point() +
  stat_smooth() +
  ggtitle("Residual plot for new fit")

# Here's what the residuals vs fitted plot looks like with our new model
ggplot(data.frame(Fitted = predict(lm.cc2),
                  Residuals = residuals(lm.cc2)),
       aes(x = Fitted, y = Residuals)) +
  geom_point() +
  stat_smooth() +
  ggtitle("Residual plot for new fit")

##########################
## UNUSUAL OBSERVATIONS ##
##########################

# why do we care? Influential data points pull the regression
# function towards them, leading to incorrect inferences 
# about the regression function.  

# we present several diagnostics here but also you should always
# just graph your data.  Use identify() to label the points!

# hat values are a measure of leverage, telling you 
# how much 'force' a particular observation exerts on the 
# placement of the regression line

hats <- hatvalues(lm.cc2)
test <- plot(hats, pch = "")
text(hats, labels = as.character(names(hats)),cex = .8)

# studentized residuals sequentially omit each observation i,
# refit the regression model, and then measure that observations
# outlyingness.  These residuals are standardized too.

stud.res <- rstudent(lm.cc2) 

# Outlyingness in both directions
par(mfrow=c(1,2))
plot(hats, stud.res, ylim = c(-2,8), pch = "", 
     main = "Influence plot", cex = .5)
text(hats, stud.res, labels = as.character(names(hats)),cex = .8)

plot(lm.cc2, 5)

# cook's distance is sometimes a handy general statistic for detection
# of influence points 

# We can get Cook's distance from
cooks_dist <- cooks.distance(lm.cc2)

par(mfrow=c(1,1))
plot(lm.cc2, 4)

# so our possible influence points are 45 and 23
cc[45,]  # this individual has the highest expenditure in the group, and relatively old
cc[23,]  # this individual has very high cc debt, and is much older
summary(cc)

# predictions: what will happen if we reduce the influence 
# of these folks in fitting the regression?


## ------------------------------------
## Cooks Distance Calculate by heand
## ------------------------------------
## Hat Matrix
X <- model.matrix(lm.cc2)
hat.mat <- X %*% solve(t(X) %*% X) %*% t(X)
hat.values <- diag(hat.mat)
uhat <- resid(lm.cc2)
uhatsq <- uhat * uhat
k <- 4
sig <- summary(lm.cc2)$sigma

stand.u <- uhat / (sig * sqrt(1 - hat.values))
cookdist <- ((stand.u ^ 2) / (k + 1)) * (hat.values / (1 - hat.values))

c <- cooks.distance(lm.cc2)
all.equal(cookdist, c)

## remember, hat value is basically a measure of how far
## from center of distribution - points of leverage
## stud. resid for outliers

## How would I remove a point of influence?
data.nolev <- cc[c(-23,-45), ] # remove points of influence

nolev.lm <- lm(ccexpend ~ income + I(income^2) + homeowner + age, data = data.nolev)
summary(nolev.lm)

##### Robust Estimation #########

## MM estimation 
lm.cc_mm <- rlm(ccexpend ~ income + I(income^2) + homeowner + age, data = cc, method = "MM")

## Comparing some output
summary(lm.cc2) # original
summary(nolev.lm) ## Leverage points removed 
summary(lm.cc_mm) ## MM estimation

